bl_info = {
    "name": "Shader Generator",
    "author": "Vera Sheleketo & Sergey Metelskiy",
    "version": (1, 0),
    "blender": (2, 69, 0),
    "location": "Properties > Meterial Tab (cycles context)",
    "description": "Parametric material generator for Cycles",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "category": "Render"}


import bpy
from bpy.props import EnumProperty, FloatProperty, IntProperty, StringProperty, BoolProperty, FloatVectorProperty
import json, os, re #Импортируем необходимые для работы внутренние модули


path = os.getcwd()+'/'+bpy.app.version_string[0:4]+'/scripts/addons/matgen/' #Путь до папки с аддоном
jObj = open(path+'presets.json','r')
presets = json.load(jObj) #Конвертируем пресеты из JSON в Python объект

def save_presets_tofile():
    global path, presets
    try:
        with open(path+'presets.json', 'w') as outfile:
            json.dump(presets, outfile, indent=4, separators=(',', ': '))
            print(json.dumps(presets, sort_keys=True, indent=4, separators=(',', ': ')))
    except:
        return False
    return True

def category_update(self, context): #Функция вызываемая на обновление категории
    tree = context.active_object.active_material.node_tree
    nodes = tree.nodes
    links = tree.links
    
    shader = None
    for node in nodes:
        if node.type == 'GROUP' and node.node_tree == bpy.data.node_groups['matgen_shader']:
            shader = node
            break
    
    output = nodes['Material Output']
    
    for link in links: #удаляем из дерева все связи
        links.remove(link)
    
    #Устанавливаем соответствующие связи по новой для выбранной категории    
    if self.category == 'Metall':
        links.new(output.inputs['Surface'], shader.outputs['Metall'])
        links.new(output.inputs['Displacement'], shader.outputs['Metall Bump'])
        return
    
    if self.category == 'Stone':
        links.new(output.inputs['Surface'], shader.outputs['Stone'])
        links.new(output.inputs['Displacement'], shader.outputs['Stone Bump'])
        return
    
    if self.category == 'Glass':
        links.new(output.inputs['Surface'], shader.outputs['Glass'])
        return
    
    if self.category == 'Translucent':
        links.new(output.inputs['Surface'], shader.outputs['Translucent'])
        return
    
    if self.category == 'Snow':
        links.new(output.inputs['Surface'], shader.outputs['Snow'])
        links.new(output.inputs['Displacement'], shader.outputs['Snow Bump'])
        return
    
    context.active_object.active_material.matgen.preset = 'Default'

def preset_update(self, context): #Функция, вызываемая на установку пресета
    preset = presets[self.category][self.preset]
    shader = None
    
    for node in self.id_data.node_tree.nodes: #Проверяем есть ли узел с шейдером
        if node.type == 'GROUP' and node.node_tree == bpy.data.node_groups['matgen_shader']:
            shader = node
            break
        
    if shader == None:
        print('No shader found in node tree')
        return
    
    for key,value in preset.items():
        try:
            shader.inputs[key].default_value = value
        except KeyError:
            print(key,'property not found in shader')
    
def getPresetList(self, context): #Функция возвращающая список всех пресетов в данной категории
    items = [('None','None','None')]
    if self.category in presets.keys() and len(presets[self.category]) > 0:
        items = []
        for preset in presets[self.category].keys():
            items.append((preset, preset, preset))
    return items

class MatGenProps(bpy.types.PropertyGroup):
    category = EnumProperty(items=[('Glass','Стекло','Glass'),
                                   ('Metall','Металл','Metall'),
                                   ('Stone','Камень','Stone'),
                                   ('Snow','Снег','Snow'),
                                   ('Translucent','Полупрозрачный','Translucent'),],
                            name='Категория', description='Категория генерируемого материала', update=category_update)
    preset = EnumProperty(name='Пресет', items=getPresetList, description='Пресет, относящийся к выбранной категории', update=preset_update)
                                   

bpy.utils.register_class(MatGenProps)
bpy.types.Material.matgen = bpy.props.PointerProperty(type=MatGenProps)

class GenerateShader(bpy.types.Operator):
    bl_idname = 'material.mat_gen'
    bl_label = 'Generate Shader'
    bl_description = 'Генерирует универсальный шейдер для активного материала'
    bl_options = {'INTERNAL'}
    
    def execute(self, context):
        if 'matgen_shader' not in bpy.data.node_groups.keys():
            with bpy.data.libraries.load(path+'shader_tree.blend', False, True) as (data_from, data_to):
                data_to.materials = ['matgen_dummy']
        mat = context.active_object.active_material
        mat.use_nodes = True
        for node in mat.node_tree.nodes:
            mat.node_tree.nodes.remove(node)
        outputNode = mat.node_tree.nodes.new('ShaderNodeOutputMaterial')
        shaderGroup = mat.node_tree.nodes.new('ShaderNodeGroup')
        shaderGroup.node_tree = bpy.data.node_groups['matgen_shader']
        mat.node_tree.links.new(outputNode.inputs['Surface'], shaderGroup.outputs['Glass'])
        #mat.node_tree.links.new(outputNode.inputs['Displacement'], shaderGroup.outputs['Bump'])
        return{'FINISHED'}

class SavePreset(bpy.types.Operator): #Оператор сохранения текущего пресета
    bl_idname = 'material.save_matgen_preset'
    bl_label = 'AddPreset'
    bl_options = {'INTERNAL'}
    bl_description = 'Сохранить текущий пресет'
    
    def execute(self, context):
        global presets
        mat = context.active_object.active_material
        newpreset = {}
        for node in mat.node_tree.nodes:
            if node.type == 'GROUP' and node.node_tree == bpy.data.node_groups['matgen_shader']:
                shader = node
                break
        for key,value in shader.inputs.items(): #Добавляем пресет
            if value.default_value.__class__.__name__ == 'bpy_prop_array': newpreset[key] = list(value.default_value)
            else: newpreset[key] = value.default_value
        presets[mat.matgen.category][mat.matgen.preset].update(newpreset)
        if save_presets_tofile():
            print('Writen to file')
            return {'FINISHED'}
        else: #Репорт, если функция сохранения пресетов возвращает False
            self.report(message='Невозможно сохранить пресеты на диск. Проверьте разрешения', type={'ERROR'})
            return {'CANCELLED'}

class AddPreset(bpy.types.Operator): # Оператор добавления нового пресета
    bl_idname = 'material.add_matgen_preset'
    bl_label = 'Добавить Новый Пресет'
    bl_options = {'INTERNAL'}
    bl_description = 'Сохранить текущие настройки в новый пресет'
    
    presetName = StringProperty(name='Имя', description='Name for new preset', default='New Name')
    
    def invoke(self, context, event): #Вызываем всплывающее диалоговое окошко
        wm = context.window_manager
        return wm.invoke_props_dialog(self)
    
    def execute(self, context):
        global presets
        mat = context.active_object.active_material
        if self.presetName == '': #Если имя не указано, то задаем дефолтное
            self.presetName = 'New Preset'
        if re.search('[^a-zA-Z0-9_ ]+', self.presetName): #Проверяем имя на недопустимые символы
            self.report(message='Имя не указано или содержит запрещенные символы', type={'ERROR'})
            return {'CANCELLED'}
        for node in mat.node_tree.nodes:
            if node.type == 'GROUP' and node.node_tree == bpy.data.node_groups['matgen_shader']:
                shader = node
                break
        newpreset = {}
        for key,value in shader.inputs.items(): #Добавляем пресет
            if value.default_value.__class__.__name__ == 'bpy_prop_array': newpreset[key] = list(value.default_value)
            else: newpreset[key] = value.default_value
        presets[mat.matgen.category][self.presetName] = newpreset
        if save_presets_tofile():
            print('Writen to file')
            #mat.matgen.category = self.presetName
            return {'FINISHED'}
        else: #Репорт, если функция сохранения пресетов возвращает False
            self.report(message='Невозможно сохранить пресеты на диск. Проверьте разрешения', type={'ERROR'})
            return {'CANCELLED'}

class DeletePreset(bpy.types.Operator): #Оператор удаления пресета
    bl_idname = 'material.delete_matgen_preset'
    bl_label = 'DeletePreset'
    bl_options = {'INTERNAL'}
    bl_description = 'Удалить выбранный пресет'
    
    def execute(self, context):
        global presets
        mat = context.active_object.active_material
        if mat.matgen.preset == 'Default': #Проверяем, не Default ли пытается удалить пользователь
            self.report(message='Пресет Default удалить нельзя', type={'ERROR'})
            return {'CANCELLED'}
        del presets[mat.matgen.category][mat.matgen.preset]
        if save_presets_tofile():
            print('Writen to file')
            mat.matgen.preset = 'Default'
            return {'FINISHED'}
        else: #Репорт, если функция сохранения пресетов возвращает False
            self.report(message='Невозможно сохранить пресеты на диск. Проверьте разрешения', type={'ERROR'})
            return {'CANCELLED'}
        mat.matgen.preset = 'Default' #После удаления устанавливаем дефолтный пресет
        return {'FINISHED'}
    
class ApplyMatToSelected(bpy.types.Operator): #Оператор, применяющий активный материал ко всем выделенным объектам
    bl_idname = 'material.apply_mat_toselected'
    bl_label = 'Apply Material To Selected'
    bl_options = {'INTERNAL'}
    bl_description = 'Применить материал к выбранным объектам'
    
    def execute(self, context):
        mat = context.active_object.active_material
        for ob in context.selected_objects:
            ob.active_material = mat
        return {'FINISHED'}

class MatGenPanel(bpy.types.Panel):
    bl_idname = 'PROP_matgen'
    bl_label = 'Shader generator'
    bl_context = 'material'
    bl_region_type = 'WINDOW'
    bl_space_type = 'PROPERTIES'
    
    @classmethod
    def poll(cls, context):
        return context.scene.render.engine == 'CYCLES'
    
    def draw(self, context):
        l = self.layout
        mat = context.active_object.active_material
        if mat == None:
            l.label('Сначала добавьте материал в активный слот')
            return
        if mat.use_nodes != True:
            l.operator('material.mat_gen', text='Generate Shader')
            return
        matgen_tree = None
        for node in mat.node_tree.nodes:
            if node.type == 'GROUP' and node.node_tree == bpy.data.node_groups['matgen_shader']:
                matgen_tree = node
                break
        if matgen_tree == None:
            l.operator('material.mat_gen', text='Генерировать шейдер')
            return
        l.prop(mat.matgen, 'category')
        r = l.row(align=True)
        r.prop(mat.matgen, 'preset')
        r.operator('material.save_matgen_preset', text='', icon='SAVE_COPY')
        r.operator('material.add_matgen_preset', text='', icon='ZOOMIN')
        r.operator('material.delete_matgen_preset', text='', icon='ZOOMOUT')
        l.split
        if mat.matgen.category == 'Glass':
            l.label('Свойства стекла')
            l.prop(matgen_tree.inputs['Color'], 'default_value', 'Цвет')
            l.prop(matgen_tree.inputs['Opacity'], 'default_value', 'Непрозрачность', slider=True)
            l.prop(matgen_tree.inputs['IOR'], 'default_value', 'Преломление')
            l.prop(matgen_tree.inputs['Roughness'], 'default_value', 'Шероховатость', slider=True)
            l.prop(matgen_tree.inputs['Absorbtion'], 'default_value', 'Поглощение')
            #l.prop(matgen_tree.inputs['Opacity'], 'default_value', 'Дисперсия', slider=True)
            l.split()
            l.prop(matgen_tree.inputs['Visibility'], 'default_value', 'Общая видимость', slider=True)
        
        if mat.matgen.category == 'Translucent':
            l.label('Свойства полупрозрачного')
            l.prop(matgen_tree.inputs['Color'], 'default_value', 'Цвет')
            l.prop(matgen_tree.inputs['Absorbtion'], 'default_value', 'Поглощение')
            l.prop(matgen_tree.inputs['Coat'], 'default_value', 'Блеск', slider=True)
            l.prop(matgen_tree.inputs['Roughness'], 'default_value', 'Шероховатость', slider=True)
            l.split()
            l.prop(matgen_tree.inputs['Visibility'], 'default_value', 'Общая видимость', slider=True)

        if mat.matgen.category == 'Metall':
            l.label('Свойства металла')
            l.prop(matgen_tree.inputs['Color'], 'default_value', 'Цвет')
            l.prop(matgen_tree.inputs['Roughness'], 'default_value', 'Шероховатость', slider=True)
            l.prop(matgen_tree.inputs['Anisotropy Mix'], 'default_value', 'Анизотропный материал', slider=True)
            l.prop(matgen_tree.inputs['Anisotropy'], 'default_value', 'Анизотропность', slider=True)
            l.prop(matgen_tree.inputs['Coat'], 'default_value', 'Доп.Блеск', slider=True)
            l.split()
            l.prop(matgen_tree.inputs['Visibility'], 'default_value', 'Общая видимость', slider=True)
            l.label('Ржавчина')
            l.prop(matgen_tree.inputs['Rust Color'], 'default_value', 'Цвет')
            l.prop(matgen_tree.inputs['Rust Strongness'], 'default_value', 'Сила', slider=True)
            l.prop(matgen_tree.inputs['Rust Amount'], 'default_value', 'Количество', slider=True)
            l.prop(matgen_tree.inputs['Rust Scale'], 'default_value', 'Масштаб')
            
        if mat.matgen.category == 'Snow':
            l.label('Свойства снега')
            l.prop(matgen_tree.inputs['Snow OverLayer'], 'default_value', 'Цвет 1')
            l.prop(matgen_tree.inputs['Snow Backfacing'], 'default_value', 'Цвет 2')
            l.prop(matgen_tree.inputs['Snow UnderLayer'], 'default_value', 'Цвет 3')
            l.prop(matgen_tree.inputs['Snow Glitter Scale'], 'default_value', 'Масштаб блесток')
            l.prop(matgen_tree.inputs['Visibility'], 'default_value', 'Общая видимость', slider=True)
            l.split()
            l.label('Неровности')
            l.prop(matgen_tree.inputs['Snow Bump Straight'], 'default_value', 'Сила')
            l.prop(matgen_tree.inputs['Snow Bump Scale'], 'default_value', 'Масштаб')
            
        if mat.matgen.category == 'Stone':
            l.label('Свойства камня')
            l.prop(matgen_tree.inputs['Stone Color1'], 'default_value', 'Цвет 1')
            l.prop(matgen_tree.inputs['Stone Color2'], 'default_value', 'Цвет 2')
            l.prop(matgen_tree.inputs['Stone Color3'], 'default_value', 'Цвет 3')
            l.prop(matgen_tree.inputs['Stone Color Scale 1'], 'default_value', 'Масштаб 1')
            l.prop(matgen_tree.inputs['Stone Color Scale 2'], 'default_value', 'Масштаб 2')
            l.prop(matgen_tree.inputs['Stone Color Distortion'], 'default_value', 'Искажение')
            l.prop(matgen_tree.inputs['Stone Color Detail'], 'default_value', 'Детали')
            l.prop(matgen_tree.inputs['Stone Color Contrast'], 'default_value', 'Контраст')
            l.prop(matgen_tree.inputs['Coat'], 'default_value', 'Блеск', slider=True)
            l.prop(matgen_tree.inputs['Roughness'], 'default_value', 'Шероховатость', slider=True)
            l.prop(matgen_tree.inputs['Visibility'], 'default_value', 'Общая видимость', slider=True)
            l.split()
            l.label('Неровности')
            l.prop(matgen_tree.inputs['Stone Bump Straight'], 'default_value', 'Сила')
            l.prop(matgen_tree.inputs['Stone Bump Scale'], 'default_value', 'Масштаб')
            l.prop(matgen_tree.inputs['Stone Bump Detail'], 'default_value', 'Детали')
            l.prop(matgen_tree.inputs['Stone Bump Contrast'], 'default_value', 'Контраст')
        l.split()
        l.operator('material.apply_mat_toselected', text='Применить к выбранным объектам')


def register():
    bpy.utils.register_class(MatGenPanel)
    bpy.utils.register_class(GenerateShader)
    bpy.utils.register_class(SavePreset)
    bpy.utils.register_class(AddPreset)
    bpy.utils.register_class(DeletePreset)
    bpy.utils.register_class(ApplyMatToSelected)
    
def unregister():
    bpy.utils.unregister_class(MatGenPanel)
    bpy.utils.unregister_class(GenerateShader)
    bpy.utils.unregister_class(SavePreset)
    bpy.utils.unregister_class(AddPreset)
    bpy.utils.unregister_class(DeletePreset)
    bpy.utils.unregister_class(ApplyMatToSelected)


if __name__ == "__main__":
    register()
